/*
** Apple Macintosh Developer Technical Support
**
** Program:	    CShell
** File:	    help.c
** Written by:  Eric Soldan
**
** Copyright  1991 Apple Computer, Inc.
** All rights reserved.
*/



/*****************************************************************************/



#include "CShell.h"				/* Get the CShell includes/typedefs, etc.	*/
#include "CShellCommon.h"		/* Get the stuff in common with rez.		*/
#include "CShell.protos"		/* Get the prototypes for CShell.			*/

#ifndef __BALLOONS__
#include <balloons.h>
#endif

#ifndef __UTILITIES__
#include "Utilities.h"
#endif



/*****************************************************************************/
/*****************************************************************************/



#pragma segment Main
void	DynamicBalloonHelp(void)
{
	WindowPtr		window, oldPort;
	FileRecHndl		frHndl;
	Point			mouseLoc, tip;
	ControlHandle	ctl;
	Rect			rct;
	short			part, message, pos;
	TEHandle		teHndl;
	HMMessageRecord	helpMessage;
	static short	lastMessage;
	static short	position[4] = {5, 6, 2, 1};

	if (gSystemVersion < 0x0700) return;
		/* The system can't support balloons. */

	if ((!HMGetBalloons()) || (!IsAppWindow(FrontWindow()))) {
		lastMessage = 0;
		return;
	}		/* Balloons have been turned off, or it isn't our window anymore. */

	HMGetBalloonWindow(&window);
	if (!window) lastMessage = 0;
		/* There is no balloon currently, so there is no last message. */

	tip = mouseLoc = GetGlobalMouse();
	part = FindWindow(mouseLoc, &window);
	if ((window != FrontWindow()) || (part != inContent)) {
		lastMessage = 0;
		return;
	}		/* We aren't over the content of the front window, so leave. */

	GetPort(&oldPort);
	frHndl = (FileRecHndl)GetWRefCon(window);
	SetPort(window);
	GlobalToLocal(&mouseLoc);

	ctl = ((WindowPeek)window)->controlList;
	while (ctl) {
		rct = (*ctl)->contrlRect;
		if (PtInRect(mouseLoc, &rct)) break;
		ctl = (*ctl)->nextControl;
	}		/* Find the control that we are over. */

	message = 0;
	if (ctl) {
		teHndl = (TEHandle)GetCRefCon(ctl);
		if (teHndl == (*frHndl)->doc.inBox)  message = 2;
		if (teHndl == (*frHndl)->doc.outBox) message = 4;
	}

	if (message) {
		if ((*frHndl)->doc.connected) --message;
		if (lastMessage != message) {	/* If this balloon isn't current balloon... */
			lastMessage = message;
			helpMessage.hmmHelpType             = khmmStringRes;
			helpMessage.u.hmmStringRes.hmmResID = rDynHelpStrings;
			helpMessage.u.hmmStringRes.hmmIndex = message;
			LocalToGlobalRect(&rct);
			pos = (tip.v > (rct.top + rct.bottom) / 2) ? 2 : 0;
			if (tip.h > (rct.left + rct.right) / 2) ++pos;
			pos = position[pos];
			SetOrigin(0, 0);
			HMShowBalloon(&helpMessage, tip, &rct, nil, 0, pos, kHMRegularWindow);
		}
	}
	else {
		if (lastMessage) HMRemoveBalloon();
		lastMessage = 0;
	}

	SetOrigin(0, 0);
	SetPort(oldPort);
}



