/*
** Apple Macintosh Developer Technical Support
**
** Program:	     MacShell
** File:	     AEUtils.c
** Written by:   Keith Rollin
** Modified by:  Eric Soldan
**
** Copyright  1990-1991 Apple Computer, Inc.
** All rights reserved.
*/



/*****************************************************************************/



#include "MacShell.h"			/* Get the MacShell includes/typedefs, etc.	*/
#include "MacShellCommon.h"		/* Get the stuff in common with rez.		*/
#include "MacShell.protos"		/* Get the prototypes for MacShell.			*/



/*****************************************************************************/



extern Boolean		gHasPPCToolbox;



/*****************************************************************************/
/*****************************************************************************/



/* DoHighLevelEvent
**
** Simply calls AEProcessAppleEvent and reports any errors.
** AEProcessAppleEvent looks in its table of registered events and sees if
** the current event is registered.  If so, it calls the routine associated
** with that event.  In our case, we set that to DispatchAppleEvent.
** DispatchAppleEvent handles it in all cases.  DispatchAppleEvent uses the
** refCon to determine which AppleEvent it is doing, since it gets them all.
*/

#pragma segment AppleEvents
void	DoHighLevelEvent(EventRecord *event)
{
	AEProcessAppleEvent(event);
}



/*****************************************************************************/



/* GetTargetInfo
**
** This function returns the zone, machine, and application name for the
** indicated target descriptor.
*/

#pragma segment AppleEvents
OSErr	GetTargetInfo(AEAddressDesc targetDesc, StringPtr zone,
					  StringPtr machine, StringPtr application)
{
	ProcessSerialNumber targetPSN;
	PortInfoRec			portInfo;
	TargetID			theTargetID;
	OSErr				err;

	zone[0]        = 0;
	machine[0]     = 0;
	application[0] = 0;
	err = noErr;

	if (targetDesc.descriptorType == typeProcessSerialNumber) {
		targetPSN = **(ProcessSerialNumber **)(targetDesc.dataHandle);
		err = GetPortNameFromProcessSerialNumber(&portInfo.name, &targetPSN);
		if (!err) pstrcpy((char *)application, (char *)portInfo.name.name);
		return(err);
	}

	if (targetDesc.descriptorType == typeTargetID) {
		theTargetID = **(TargetID **)(targetDesc.dataHandle);
		switch (theTargetID.location.locationKindSelector) {
			case ppcNoLocation:
				break;
			case ppcNBPLocation:
				pstrcpy((char *)zone,    (char *)theTargetID.location.u.nbpEntity.zoneStr);
				pstrcpy((char *)machine, (char *)theTargetID.location.u.nbpEntity.objStr);
				break;
			case ppcNBPTypeLocation:
				break;
		}
		pstrcpy((char *)application, (char *)theTargetID.name.name);
		return(noErr);
	}

	return(errAEWrongDataType);
}



/*****************************************************************************/



/* MakeTarget
**
** Creates a TargetID.
**
** If sendDirect is TRUE, the target is specified by setting a
** ProcessSerialNumber to kCurrentProcess.  This has the advantage of sending
** the message directly to ourselves, bypassing ePPC and gaining about a 10-15x
** speed improvement.  If sendDirect is FALSE, we see if we have the
** PPCToolBox.  If not, then we are forced to do a direct send.  If we do have
** the PPCToolbox, then we call PPCBrowser.  We then look at the reply, and
** factor in the mode we are going to use in AESend.  If that mode is
** kAEWaitReply and the user selected us as the target, we have to turn that
** into a direct send.  This is because the AppleEvent Manager will otherwise
** post the event as a high-level event.  However, we are busy waiting for a
** reply, not looking for events, so we'll hang.  We avoid this by forcing a
** direct send.
*/

#pragma segment AppleEvents
OSErr	MakeTarget(AEAddressDesc *target, Boolean sendDirect, short replyMode,
				   Str255 prompt, Str255 applListLabel,
				   PPCFilterProcPtr portFilter,
				   char *theLocNBPType)
{
	OSErr					err;
	ProcessSerialNumber 	targetPSN;
	ProcessSerialNumber 	myPSN;
	TargetID				theTargetID;
	Boolean					sendingToSelf;

	static LocationNameRec	location;
	static PortInfoRec		portInfo;
	static Boolean			defaultOK = false;

	err = noErr;	/* Make sure we do the code for the second main if. */

	target->dataHandle = nil;
		/* Assume we will fail and nil this descriptor out. */

	if (!sendDirect) {
		if (!gHasPPCToolbox)
			sendDirect = true;	/* No tools to send with, so send direct. */

		else {		/* We are not sending to self. */
					/* sendDirect is false.		   */
			err = PPCBrowser(
				prompt,			/* Browse dialog box prompt.		   */
				applListLabel,	/* The 'programs' list title.		   */
				defaultOK,		/* Initially false.					   */
				&location,		/* Correct if defaultOK is true.	   */
				&portInfo,		/* Correct if defaultOK is true.	   */
				portFilter,		/* Port filtering.					   */
				(StringPtr)theLocNBPType	/* List ports of this type */
			);

			if (!err) {					/* If user didn't cancel... */
				defaultOK = true;		/* Default to the same port next time. */
				if (replyMode == kAEWaitReply) {
					/* Sender wants a reply and will be waiting... */

					sendingToSelf = false;
						/* Assume that we aren't sending to ourselves. */

					if (!location.locationKindSelector) {
						/* Hey, we are sending to ourselves! */

						err = GetProcessSerialNumberFromPortName(
								&portInfo.name, &targetPSN);
						if (!err) {
							GetCurrentProcess(&myPSN);
							err = SameProcess(&targetPSN, &myPSN, &sendingToSelf);
						}
					}

					if (sendingToSelf) sendDirect = true;

				}
			}
		}
	}

	if (!err) {
		if (sendDirect) {
			/* Finally, we get to the point... */

			targetPSN.highLongOfPSN = 0;
			targetPSN.lowLongOfPSN = kCurrentProcess;
				/* Process serial # is equal to kCurrentProcess.  This
				** bypasses ePPC and speeds up things considerably. */

			err = AECreateDesc(
				typeProcessSerialNumber,	/* Standard PSN descriptor type. */
				(Ptr)&targetPSN,			/* "No ePPC" process serial #.	 */
				sizeof(targetPSN),			/* Size of data (2 longs).		 */
				target						/* Wherefore art thou desc.		 */
			);
		}
		else {
			theTargetID.location = location;
			theTargetID.name     = portInfo.name;
				/* The fields sessionID does not need to be filled in now.
				** The sessionID is returned when you actually connect to
				** a port.  You can then use the sessionID from that point
				** on to improve speed.
				**
				** You also don't need to fill in the recvrName field at this
				** point.  This is filled in, again, when the session is
				** actually established.
				**
				** The amount of data for a non-us target is bigger.
				** We need the whole dealie for our target, since
				** it is out on the net somewhere. */

			err = AECreateDesc(
				typeTargetID,			/* Standard target descriptor type. */
				(Ptr)&theTargetID,		/* The data for the descriptor.		*/
				sizeof(theTargetID),	/* Size of the data.				*/
				target					/* Wherefore art thou desc.			*/
			);
		}
	}
	return (err);
}



/*****************************************************************************/



/* MissedAnyParameters
**
** Used to check for any unread required parameters. Returns true if we
** missed at least one.
*/

#pragma segment AppleEvents
Boolean	MissedAnyParameters(AppleEvent *message)
{
	OSErr		err;
	DescType	ignoredActualType;
	AEKeyword	missedKeyword;
	Size		ignoredActualSize;
	EventRecord	event;

	err = AEGetAttributePtr(	/* SEE IF PARAMETERS ARE ALL USED UP.		  */
		message,				/* AppleEvent to check.						  */
		keyMissedKeywordAttr,	/* Look for unread parameters.				  */
		typeKeyword,			/* So we can see what type we missed, if any. */
		&ignoredActualType,		/* What is would have been if not coerced.	  */
		(Ptr)&missedKeyword,	/* Data area.  (Keyword not handled.)		  */
		sizeof(missedKeyword),	/* Size of data area.						  */
		&ignoredActualSize		/* Actual data size.						  */
	);

/* No error means that we found some unused parameters. */

	if (err == noErr) {
		event.message = *(long *) &ignoredActualType;
		event.where = *(Point *) &missedKeyword;
		err = errAEEventNotHandled;
	}

/* errAEDescNotFound means that there are no more parameters.  If we get
** an error code other than that, flag it.
*/

	return(err != errAEDescNotFound);
}



