/*
** Apple Macintosh Developer Technical Support
**
** Program:	    MacShell
** File:	    docursor.c
** Written by:  Eric Soldan
**
** Copyright  1990-1991 Apple Computer, Inc.
** All rights reserved.
*/



/*****************************************************************************/



#include "MacShell.h"			/* Get the MacShell includes/typedefs, etc.	*/
#include "MacShellCommon.h"		/* Get the stuff in common with rez.		*/
#include "MacShell.protos"		/* Get the prototypes for MacShell.			*/

#ifndef __TEXTEDITCONTROL__
#include "TextEditControl.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifdef THINK_C
#include "Utilities.h"
#else
#ifndef __UTILITIES__
#include <Utilities.h>
#endif
#endif



/*****************************************************************************/



RgnHandle	gCursorRgn;
	/* The current cursor region.  The initial cursor region is an empty
	** region, which will cause WaitNextEvent to generate a mouse-moved
	** event, which will cause us to set the cursor for the first time.
	*/

Cursor	gCursor, *gCursorPtr;
	/* The current cursor that applies to gCursorRgn.  These values
	** are here to shorten the re-processing time for determining the
	** correct cursor after an event.  This is specifically so that characters
	** can be typed into the TextEdit control faster.  If we spend a great
	** deal of time per-event recalculating the cursor region, text entry for
	** the TextEdit control slows down considerably.  If you want to override
	** the time savings because you are changing the cursor directly, either
	** set gCursorPtr to nil, or call DoSetCursor to set the cursor.
	** DoSetCursor simply sets gCursorPtr to nil, as well as setting
	** the cursor.
	*/



/*****************************************************************************/
/*****************************************************************************/



/* Handle the cursor changes, based on if an AppleEvent is involved, or
** depending on the location of the cursor.  This also calculates the region
** where the current cursor resides (for WaitNextEvent).  If the mouse is
** ever outside of that region, an event would be generated, causing this
** function to be called, allowing us to change the region to the region
** the mouse is currently in.  The only other cursor management in this sample
** is for operations such as pulling down a menu.  Just prior to pulling down
** the menu, the arrow cursor is set.  This prevents any latencies in cursor
** update to cause a non-arrow cursor to be used in the menus.  This technique
** should be carried throughout the application.
*/

#pragma segment Main
void	DoCursor(void)
{
	WindowPtr		window;
	Point			mouseLoc;
#if MACSHELL_VERSION
	RgnHandle		rgn;
	Rect			outBoxRct, teViewRct, contRct;
	TEHandle		outBox, teHndl;
	FileRecHndl		frHndl;
	ControlHandle	viewCtl;
	Cursor			**crsrHndl;
	Point			contOrg;
#endif

	mouseLoc = GetGlobalMouse();

	if ((!gInBackground) && (!IsDAWindow(window = FrontWindow()))) {

		if (IsAppWindow(window)) {

			if (gCursorPtr) {							/* Do we already have a cursor... */
				if (PtInRgn(mouseLoc, gCursorRgn)) {	/* Are we still in the cursor area... */
					SetCursor(gCursorPtr);				/* Then set it to that. */
					return;
				}
			}

#if MACSHELL_VERSION

			/* For the MacShell sample, we display an i-beam cursor when over the outbox. */

			frHndl    = (FileRecHndl)GetWRefCon(window);
			viewCtl   = CTEViewFromTE(outBox = (*frHndl)->doc.outBox);
			outBoxRct = (*viewCtl)->contrlRect;		/* Local coordinates of outbox. */
			GetContentOrigin(window, &contOrg);		/* Scroll position of window. */
			OffsetRect(&outBoxRct, -contOrg.h, -contOrg.v);
				/* By offseting the outbox rect by the amount scrolled, we now have
				** the local position of the outbox within the potentially scrolled
				** window content. */
			GetContentRect(window, &contRct);
				/* This returns the content portion of the window in local coordinates,
				** less document scrollbar area. */
			SectRect(&outBoxRct, &contRct, &outBoxRct);
				/* Part of the outbox still visible after scrolling. */
			LocalToGlobalRect(&outBoxRct);
				/* The outbox rect, in global coordinates. */

			if (CTETargetInfo(&teHndl, &teViewRct) == window) {
				/* If target TextEdit control belongs to this window... */

				if ((teHndl == outBox) && (PtInRect(mouseLoc, &outBoxRct))) {
					/* If target TextEdit control is the outbox, and the cursor
					** is over the visible part of the outbox... */

					RectRgn(gCursorRgn, &outBoxRct);
						/* Set the cursor region to this rect. */

					if (crsrHndl = GetCursor(ibeamCursor)) {
						gCursor = **crsrHndl;	/* Non-handle copy keeps it from moving. */
						SetCursor(gCursorPtr = &gCursor);
					}		/* Set the current cursor to the i-beam cursor resource. */
					return;
				}
			}
			SetRectRgn(gCursorRgn, kExtremeNeg, kExtremeNeg, kExtremePos, kExtremePos);
			rgn = NewRgn();
			RectRgn(rgn, &outBoxRct);
			DiffRgn(gCursorRgn, rgn, gCursorRgn);
			DisposeRgn(rgn);

#endif

		}

		DoSetCursor(&qd.arrow);
	}

	else {
		SetRectRgn(gCursorRgn, kExtremeNeg, kExtremeNeg, kExtremePos, kExtremePos);
		gCursorPtr = nil;
	}
}



/*****************************************************************************/



#pragma segment Main
void	DoSetCursor(Cursor *cursor)
{
	if (cursor) SetCursor(cursor);
	gCursorPtr = nil;
	if (!cursor) DoCursor();
}



