/*
** Apple Macintosh Developer Technical Support
**
** Program:     MacShell
** File:        file2.c
** Written by:  Eric Soldan
**
** Copyright  1990-1991 Apple Computer, Inc.
** All rights reserved.
*/



/*****************************************************************************/



#include "MacShell.h"			/* Get the MacShell includes/typedefs, etc.	*/
#include "MacShellCommon.h"		/* Get the stuff in common with rez.		*/
#include "MacShell.protos"		/* Get the prototypes for MacShell.			*/

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifdef THINK_C
#include "Utilities.h"
#else
#ifndef __UTILITIES__
#include <Utilities.h>
#endif
#endif



/*****************************************************************************/
/*****************************************************************************/



/* AppFreeDocument
**
** Frees up any application-specific memory in the document.  The document
** may have a bunch of handles off the main handle of the document.  This
** is where they are freed.  AppDisposeDocument calls this prior to releasing
** the ram for the main handle of the document, so release everything else
** here, or you will have a memory leak.
*/

#pragma segment File
void	AppFreeDocument(FileRecHndl frHndl)
{
#if MACSHELL_VERSION

	Handle	textHndl;

	if (textHndl = (*frHndl)->doc.textHndl) {
		DisposHandle(textHndl);
		(*frHndl)->doc.textHndl = nil;
	}

#else
#pragma unused (frHndl)
#endif

}



/*****************************************************************************/



/* AppInitDocument
**
** Do any additional document initialization here.
*/

#pragma segment File
OSErr	AppInitDocument(FileRecHndl frHndl)
{

#ifdef MACSHELL_VERSION

	(*frHndl)->connect.connected = false;
	(*frHndl)->doc.version       = kVersion;
	(*frHndl)->doc.printRecValid = false;

#endif

#if MACSHELL_VERSION

	(*frHndl)->doc.textHndl = nil;

#endif

	return(noErr);
}



/*****************************************************************************/



/* AppReadDocument
**
** Reads in specified file into the data portion of a document handle.
*/

#pragma segment File
OSErr	AppReadDocument(FileRecHndl frHndl)
{
	short	fileRefNum;
	OSErr	err;
#ifdef MACSHELL_VERSION
	char	hstate;
	Ptr		ptr1, ptr2;
	long	count;
#endif
#if MACSHELL_VERSION
	Handle	textHndl;
#endif

	fileRefNum = (*frHndl)->fileState.refNum;

	/* This sample code section is separated from the below section because
	** you may want to keep this sample, whereas you probably want to replace
	** the next part.  The general print and version information is read in
	** here, and the way that it is handled works well enough that you probably
	** want to keep it.  The next section however, is very application specific.
	** Also see AppWriteDocument. */

	err = SetFPos(fileRefNum, fsFromStart, 0);
		/* Set the file position to the beginning of the file. */

#ifdef MACSHELL_VERSION

	if (!err) {		/* Read header info from file. */
		hstate = LockHandleHigh((Handle)frHndl);
		ptr1   = (Ptr)&((*frHndl)->doc);
		ptr2   = (Ptr)&((*frHndl)->doc.endVersPrintInfo);
		count  = (long)ptr2 - (long)ptr1;
		err    = FSRead(fileRefNum, &count, ptr1);
		HSetState((Handle)frHndl, hstate);
		if ((*frHndl)->doc.version != kVersion) err = kWrongVersion;
	}

#endif

#if MACSHELL_VERSION

	if (!err) {		/* Read TextEdit text from file. */
		textHndl = NewHandle(32768);
		if (!(err = MemError())) {
			count = 32768;
				/* The size of the text isn't saved to disk.  This is the maximum
				** that a TextEdit record can accept. */
			hstate = LockHandleHigh(textHndl);
			err    = FSRead(fileRefNum, &count, *textHndl);
			HSetState(textHndl, hstate);
			if (err == eofErr) err = noErr;
			if (err) count = 0;
			SetHandleSize(textHndl, count);
				/* Set the handle to the actual size of the text on disk. */
			(*frHndl)->doc.textHndl = textHndl;
		}
	}

#endif

	return(err);
}



/*****************************************************************************/



/* AppWriteDocument
**
** Writes the data portion of a document handle to the specified file.
*/

#pragma segment File
OSErr	AppWriteDocument(FileRecHndl frHndl)
{
	short		fileRefNum;
	OSErr		err;
#ifdef MACSHELL_VERSION
	char		hstate;
	Ptr			ptr1, ptr2;
	long		count, fpos;
#endif
#if MACSHELL_VERSION
	TEHandle	te;
	Handle		textHndl;
#endif

	fileRefNum = (*frHndl)->fileState.refNum;

	err = SetFPos(fileRefNum, fsFromStart, 0);
		/* Set the file position to the beginning of the file. */

	/* This sample code section is separated from the below section because
	** you may want to keep this sample, whereas you probably want to replace
	** the next part.  The general print and version information is read in
	** here, and the way that it is handled works well enough that you probably
	** want to keep it.  The next section however, is very application specific.
	** Also see AppReadDocument. */

#ifdef MACSHELL_VERSION

	if (!err) {		/* Write header info to file. */
		hstate = LockHandleHigh((Handle)frHndl);
		ptr1   = (Ptr)&((*frHndl)->doc);
		ptr2   = (Ptr)&((*frHndl)->doc.endVersPrintInfo);
		count  = (long)ptr2 - (long)ptr1;
		err    = FSWrite(fileRefNum, &count, ptr1);
		HSetState((Handle)frHndl, hstate);
	}

#endif

#if MACSHELL_VERSION

	if (!err) {		/* Write out-box TextEdit control text to file. */
		te       = (*frHndl)->doc.outBox;
		textHndl = (*te)->hText;
		count    = (*te)->teLength;
		hstate   = LockHandleHigh(textHndl);
		err      = FSWrite(fileRefNum, &count, *textHndl);
		HSetState(textHndl, hstate);
	}

#endif

	if (!err) {
		err = GetFPos(fileRefNum, &fpos);
		if (!err) err = SetEOF(fileRefNum, fpos);
	}

	return(err);
}



/*****************************************************************************/



#pragma segment File
OSErr	AppDuplicateDocument(FileRecHndl oldFrHndl, FileRecHndl *newFrHndl)
{
#if MACSHELL_VERSION
	TEHandle	te;
	Handle		oldHndl, newHndl;
	long		size;
#else
#pragma unused (oldFrHndl)
#endif
	OSErr		err;

	err = AppNewDocument(newFrHndl, (*oldFrHndl)->fileState.sfType);
	if (!err) {

#if MACSHELL_VERSION

		te      = (*oldFrHndl)->doc.outBox;
		oldHndl = (*te)->hText;
		size    = (*te)->teLength;
		if (newHndl = NewHandle(size)) {
			(**newFrHndl)->doc.textHndl = newHndl;
			BlockMove(*oldHndl, *newHndl, size);
		}
		else err = memFullErr;

#endif

	}

	return(err);
}



